-- Makes the ingame scene, adds the various components
newScene('ingame', {250,245,229})

scenes:setInit('ingame', function() 
	player = {}
	player.img = love.graphics.newImage('/img/ingame/E.png')
	player.w = player.img:getWidth()
	player.h = player.img:getHeight()
	player.x = SCREEN.w/2
	player.y = SCREEN.h/2
	player.rotation = 0
	player.dRad = math.pi/2
	player.speed = 150
	player.shots = {}

	enemies = {}
	waveNum = 1
	print('Wave number: '..waveNum)

	-- various constructors for the enemy types
	cnstr = {}

	cnstr.o = {}
	cnstr.o.type = 'o'
	cnstr.o.img = love.graphics.newImage('/img/ingame/o.png')
	cnstr.o.w = cnstr.o.img:getWidth()
	cnstr.o.h = cnstr.o.img:getHeight()
	cnstr.o.speed = 64
	cnstr.o.dRad = math.pi
	cnstr.o.radius = 16

	cnstr.e = {}
	cnstr.e.type = 'e'
	cnstr.e.img = love.graphics.newImage('/img/ingame/e.png')
	cnstr.e.w = cnstr.e.img:getWidth()
	cnstr.e.h = cnstr.e.img:getHeight()
	cnstr.e.speed = 96
	cnstr.e.dRad = math.pi*4
	cnstr.e.radius = 16

	cnstr.M = {}
	cnstr.M.type = 'M'
	cnstr.M.img = love.graphics.newImage('/img/ingame/M.png')
	cnstr.M.w = cnstr.M.img:getWidth()
	cnstr.M.h = cnstr.M.img:getHeight()
	cnstr.M.speed = 56
	cnstr.M.dRad = math.pi/2

	spawnEnemies()
	end)

scenes:setUpdate('ingame', function(dt)
	function love.keypressed(k)
		if k == 'escape' then
			scenes:setActive('menu')
		--elseif k == 'f2' then
		--	love.load()
		end
	end

	function love.keyreleased(k)
		if k == ' ' then
			shoot()
		end
	end

	local remShots, remEnemies = {}, {}

	if love.keyboard.isDown('a', 'left') then
		if player.rotation <= 0 then player.rotation = math.pi*2 end
		player.rotation = player.rotation - player.dRad*dt
	end
	if love.keyboard.isDown('d', 'right') then
		if player.rotation >= math.pi*2 then player.rotation = 0 end
		player.rotation = player.rotation + player.dRad*dt
	end
	if love.keyboard.isDown('w','up') then
		dx, dy = update_obj(player.speed*dt, player.rotation)
		player.x = player.x + dx
		player.y = player.y + dy
	end

	if #player.shots ~= 0 then
		for i,v in ipairs(player.shots) do
			dx, dy = update_obj(v.speed*dt, v.rotation)
			v.x = v.x + dx
			v.y = v.y + dy
			if v.x - v.radius*2 < 0 or
				v.x > SCREEN.w or
				v.y - v.radius*2 < 0 or
				v.y > SCREEN.h then
				table.insert(remShots, i)
			end
			local collision = chk_shot(v)
			if collision then
				table.insert(remEnemies, collision)
				table.insert(remShots, i)
			end
		end
	end

	if #remShots ~= 0 then
		for i,v in ipairs(remShots) do
			table.remove(player.shots, i)
		end
	end

	for i,v in ipairs(remEnemies) do
		table.remove(enemies, v)
	end

	if #enemies == 0 then
		waveNum = waveNum + 1
		print('Wave number: '..waveNum)
		spawnEnemies()
	end

	for i,v in ipairs(enemies) do
		update_unit(dt, v)
	end
end)

scenes:setDraw('ingame', function()
	love.graphics.draw(player.img, player.x, player.y, player.rotation, 1,1, player.w/2, player.h/2)
	love.graphics.setColor(0,0,0)
	for i,v in ipairs(player.shots) do
		love.graphics.circle('fill',v.x,v.y,v.radius)
	end
	love.graphics.setColor(255,255,255)
	for i,v in ipairs(enemies) do
		love.graphics.draw(v.img,v.x,v.y,v.rotation,1,1,v.w/2,v.h/2)
	end
	end)

-- Various updating functions below

function update_obj(hyp, rot)
	local x, y = 0, 0
	if rot == 0 then
		x, y = 0, -hyp
	elseif rot == math.pi/2 then
		x, y = hyp, 0
	elseif rot == math.pi then
		x, y = 0, hyp
	elseif rot == math.pi*3/2 then
		x, y = -hyp, 0
	else
		if rot > 0 and rot < math.pi/2 then
			x = hyp*math.sin(rot)
			y = -(hyp*math.cos(rot))
		elseif rot > math.pi/2 and rot < math.pi then
			rot = rot-math.pi/2
			x = hyp*math.cos(rot)
			y = hyp*math.sin(rot)
		elseif rot > math.pi and rot < math.pi*3/2 then
			rot = rot-math.pi
			x = -(hyp*math.sin(rot))
			y = hyp*math.cos(rot)
		elseif rot > math.pi*3/2 and rot < math.pi*2 then
			rot = rot-math.pi*3/2
			x = -(hyp*math.cos(rot))
			y = -(hyp*math.sin(rot))
		end
	end
	return x, y
end

function shoot()
	local shot = {}
	shot.rotation = player.rotation+math.pi/2
	if shot.rotation >= math.pi*2 then
		shot.rotation = shot.rotation - math.pi*2
	end
	shot.x = player.x
	shot.y = player.y
	shot.speed = 200
	shot.radius = 3
	table.insert(player.shots, shot)
end

function newEnemy(Type)
	local enemy = {}
	side = math.random(1,4)
	if side == 1 or side == 3 then
		enemy.y = math.random(0,SCREEN.h-cnstr.o.h)
	elseif side == 2 or side == 4 then
		enemy.x = math.random(0,SCREEN.w-cnstr.o.w)
	end
	if side == 1 then enemy.x = 0-cnstr[Type].w
	elseif side == 3 then enemy.x = SCREEN.w
	elseif side == 2 then enemy.y = 0-cnstr[Type].h
	elseif side == 4 then enemy.y = SCREEN.h
	end
	enemy.rotation = math.random()*math.pi*2
	for k,v in pairs(cnstr[Type]) do
		enemy[k] = v
	end
	table.insert(enemies, enemy)
end

function update_unit(dt, unit)
	local dx, dy, bearing = 0, 0, 0
	dx = unit.x-player.x
	dy = unit.y-player.y
	local mdx, mdy = math.abs(dx), math.abs(dy)
	if dx == 0 and dy == 0 then
		--end_game()/lose_life()/something...
		return
	elseif dx == 0 and dy < 0 then
		bearing = math.pi
	elseif dx == 0 and dy > 0 then
		bearing = 0
	elseif dx < 0 and dy == 0 then
		beaing = math.pi/2
	elseif dx > 0 and dy == 0 then
		bearing = math.pi*3/2
	else
		if dx < 0 and dy < 0 then
			bearing = math.pi/2 + math.atan(mdy/mdx)
		elseif dx < 0 and dy > 0 then
			bearing = math.atan(mdx/mdy)
		elseif dx > 0 and dy < 0 then
			bearing = math.pi + math.atan(mdx/mdy)
		elseif dx > 0 and dy > 0 then
			bearing = math.pi*3/2 + math.atan(mdy/mdx)
		end
	end

	if unit.type == 'o' or unit.type == 'M' then
		local unitRot = unit.rotation
		if unitRot < bearing then
			unitRot = unitRot + math.pi*2
			unitRot = unitRot - bearing
		else
			unitRot = unitRot - bearing
		end
		if unitRot < math.pi then
			rotate_unit(dt, 'ccw', unit)
		elseif unitRot > math.pi then
			rotate_unit(dt, 'cw', unit)
		elseif bearing == math.pi then
			if math.random(0,1) == 1 then
				rotate_unit(dt, 'cw', unit)
			else
				rotate_unit(dt, 'ccw', unit)
			end
		end
		local dx, dy = update_obj(unit.speed*dt, unit.rotation)
		unit.x = unit.x + dx
		unit.y = unit.y + dy

	elseif unit.type == 'e' then
		rotate_unit(dt, 'cw', unit)
		local dx, dy = update_obj(unit.speed*dt, bearing)
		unit.x = unit.x + dx
		unit.y = unit.y + dy
	end
end

function rotate_unit(dt, dir, unit)
	if dir == 'cw' then
		if (unit.rotation+unit.dRad*dt) > math.pi*2 then
			unit.rotation = unit.rotation - math.pi*2
		end
		unit.rotation = unit.rotation + unit.dRad*dt
	elseif dir == 'ccw' then
		if (unit.rotation-unit.dRad*dt) < 0 then
			unit.rotation = unit.rotation + math.pi*2
		end
		unit.rotation = unit.rotation - unit.dRad*dt
	end
end

function chk_shot(shot)
	--local enemyArea, shotArea = 0, 0
	local val = false
	for i,v in ipairs(enemies) do
		if v.radius then
			--enemyArea = math.pi*2*v.radius
			local dx, dy = v.x-shot.x, v.y-shot.y
			if math.sqrt(dx^2+dy^2) <= v.radius+shot.radius then
				val = i
			end
		--else
			--enemyArea = enemy.w*enemy.h
		else
			if v.x-v.w/2 < shot.x+shot.radius-1 and
				v.y-v.h/2 < shot.y+shot.radius and
				shot.x-shot.radius-1 < v.x+v.w/2 and
				shot.y-shot.radius-1 < v.y+v.h/2 then
					val = i
			end
		end
		
	end
	return val
end

function spawnEnemies()
	local spawnNum = waveNum
	if waveNum > 5 then spawnNum = 5 end
	for i = 1,spawnNum do
		newEnemy('o')
	end
	if waveNum > 5 then
		spawnNum = waveNum - 5
		if waveNum > 10 then spawnNum = 5 end
		for i = 1,spawnNum do
			newEnemy('e')
		end
	end
	if waveNum > 10 then
		spawnNum = waveNum - 10
		if waveNum > 15 then spawnNum = 5 end
		for i = 1,spawnNum do
			newEnemy('M')
		end
	end
end
